@@ -73,6 +73,14 @@ class TCP_UDP(COMM):
         }
         self.serverModeSelectedClient = None # None means all clients, or ip:port string
 
+    def disconnect(self):
+        if self.isConnected():
+            self.openCloseSerial()
+
+    def onDel(self):
+        if self.isConnected():
+            self.openCloseSerial()
+
     def __del__(self):
         try:
             self.status = ConnectionStatus.CLOSED
@@ -311,7 +319,6 @@ class TCP_UDP(COMM):
             except Exception:
                 text = "".join(re.findall('[\d\.]*', text))
                 self.autoReconnectIntervalEdit.setText(text)
-                
 
     def openCloseSerial(self):
         if self.busy:
@@ -338,7 +345,7 @@ class TCP_UDP(COMM):
                     except Exception:
                         pass
                     self.conn = None
-                for _, conn in self.serverModeClientsConns.items():
+                for k, conn in self.serverModeClientsConns.items():
                     try:
                         conn.close()
                     except Exception:
@@ -357,6 +364,7 @@ class TCP_UDP(COMM):
                         if not target:
                             raise Exception(_("Target error") + ": " + self.config["target"][0])
                         print("-- connect", target)
+                        self.onConnectionStatus.emit(ConnectionStatus.CONNECTING, "")
                         self.conn = socket.socket()
                         self.conn.connect(target)
                         self.status = ConnectionStatus.CONNECTED
@@ -475,6 +483,7 @@ class TCP_UDP(COMM):
                     target = self.checkTarget(self.config["target"][0])
                     if not target:
                         raise Exception(_("Target error") + ": " + self.config["target"][0])
+                    self.onConnectionStatus.emit(ConnectionStatus.CONNECTING, "")
                     conn = socket.socket()
                     conn.connect(target)
                     conn.settimeout(0.1)
@@ -490,6 +499,7 @@ class TCP_UDP(COMM):
                 continue
             try:
                 # length = max(1, self.conn.in_waiting)
+                flush = True
                 try:
                     if protocolIsTcp:
                         data = conn.recv(4096)
@@ -505,12 +515,14 @@ class TCP_UDP(COMM):
                 except socket.timeout:
                     data = None
                 if data:
+                    if len(data) > 4096:
+                        flush = False
                     t = time.time()
                     # if length == 1 and not buffer: # just start receive
                     #     buffer += data
                     #     continue
                     buffer += data
-                if buffer and (time.time() - t > 0.001): # no new data in 1ms
+                if flush or (buffer and (time.time() - t > 0.001)): # no new data in 0.1ms
                     try:
                         self.onReceived(buffer)
                     except Exception as e:
