@@ -5,12 +5,16 @@ try:
     from i18n import _, tr
     import version
     import utils, utils_ui
+    from conn.base import ConnectionStatus
+    from widgets import statusBar
 except ImportError:
     from COMTool import parameters,helpAbout,autoUpdate, utils, utils_ui
     from COMTool.Combobox import ComboBox
     from COMTool import i18n
     from COMTool.i18n import _, tr
     from COMTool import version
+    from COMTool.conn.base import ConnectionStatus
+    from COMTool.widgets import statusBar
 
 try:
     from base import Plugin_Base
@@ -39,7 +43,6 @@ class Plugin(Plugin_Base):
     # vars set by caller
     send = None              # send(data_bytes=None, file_path=None)
     hintSignal = None       # hintSignal.emit(title, msg)
-    clearCountSignal = None  # clearCountSignal.emit()
     configGlobal = {}
     # other vars
     connParent = "main"
@@ -48,7 +51,6 @@ class Plugin(Plugin_Base):
     name = _("Send Receive")
     #
     receiveUpdateSignal = pyqtSignal(str, list, str) # head, content, encoding
-    sendFileOkSignal = pyqtSignal(bool, str)
     receiveProgressStop = False
     receivedData = []
     lock = threading.Lock()
@@ -57,13 +59,26 @@ class Plugin(Plugin_Base):
     lastBg = None
     defaultColor = None
     defaultBg = None
+    help = '''{}<br>
+<b style="color:#ef5350;"><kbd>F11</kbd></b>: {}<br>
+<b style="color:#ef5350;"><kbd>Ctrl+Enter</kbd></b>: {}<br>
+<b style="color:#ef5350;"><kbd>Ctrl+L</kbd></b>: {}<br>
+<b style="color:#ef5350;"><kbd>Ctrl+K</kbd></b>: {}<br>
+'''.format(
+        _('Shortcut:'),
+        _('Full screen'),
+        _('Send data'),
+        _('Clear Send Area'),
+        _('Clear Receive Area')
+    )
 
-    def onInit(self, config, plugins):
-        super().onInit(config, plugins)
+    def onInit(self, config):
+        super().onInit(config)
         self.keyControlPressed = False
         self.isScheduledSending = False
         self.config = config
         default = {
+            "version": 1,
             "receiveAscii" : True,
             "receiveAutoLinefeed" : False,
             "receiveAutoLindefeedTime" : 200,
@@ -85,8 +100,7 @@ class Plugin(Plugin_Base):
             if not k in self.config:
                 self.config[k] = default[k]
 
-    def onWidgetMain(self, parent, rootWindow):
-        self.rootWindow = rootWindow
+    def onWidgetMain(self, parent):
         self.mainWidget = QSplitter(Qt.Vertical)
         # widgets receive and send area
         self.receiveArea = QTextEdit()
@@ -121,6 +135,7 @@ class Plugin(Plugin_Base):
         self.sendButton.clicked.connect(self.onSendData)
         self.clearReceiveButtion.clicked.connect(self.clearReceiveBuffer)
         self.receiveUpdateSignal.connect(self.updateReceivedDataDisplay)
+        self.sendHistory.activated.connect(self.onSendHistoryIndexChanged)
 
         return self.mainWidget
 
@@ -205,10 +220,9 @@ class Plugin(Plugin_Base):
         self.sendSettingsAppendNewLine.clicked.connect(lambda: self.bindVar(self.sendSettingsAppendNewLine, self.config, "sendAutoNewline"))
         self.sendSettingsEscape.clicked.connect(lambda: self.bindVar(self.sendSettingsEscape, self.config, "sendEscape"))
         self.sendSettingsCRLF.clicked.connect(lambda: self.bindVar(self.sendSettingsCRLF, self.config, "useCRLF"))
-        self.sendHistory.activated.connect(self.onSendHistoryIndexChanged)
         self.receiveSettingsColor.clicked.connect(self.onSetColorChanged)
-        self.receiveSettingsAutoLinefeedTime.textChanged.connect(lambda: self.bindVar(self.receiveSettingsAutoLinefeedTime, self.config, "receiveAutoLindefeedTime", vtype=int, vErrorMsg=_("Auto line feed value error, must be integer")))
-        self.sendSettingsScheduled.textChanged.connect(lambda: self.bindVar(self.sendSettingsScheduled, self.config, "sendScheduledTime", vtype=int, vErrorMsg=_("Timed send value error, must be integer")))
+        self.receiveSettingsAutoLinefeedTime.textChanged.connect(lambda: self.bindVar(self.receiveSettingsAutoLinefeedTime, self.config, "receiveAutoLindefeedTime", vtype=int, vErrorMsg=_("Auto line feed value error, must be integer"), emptyDefault = "200"))
+        self.sendSettingsScheduled.textChanged.connect(lambda: self.bindVar(self.sendSettingsScheduled, self.config, "sendScheduledTime", vtype=int, vErrorMsg=_("Timed send value error, must be integer"), emptyDefault = "300"))
         self.sendSettingsScheduledCheckBox.clicked.connect(lambda: self.bindVar(self.sendSettingsScheduledCheckBox, self.config, "sendScheduled"))
         return widget
 
@@ -277,7 +291,6 @@ class Plugin(Plugin_Base):
         self.funcWidget.setLayout(sendFunctionalLayout)
         # event
         self.sendFileButton.clicked.connect(self.sendFile)
-        self.sendFileOkSignal.connect(self.onSentFile)
         self.saveLogCheckbox.clicked.connect(self.setSaveLog)
         self.logFileBtn.clicked.connect(self.selectLogFile)
         self.openFileButton.clicked.connect(self.selectFile)
@@ -286,6 +299,10 @@ class Plugin(Plugin_Base):
         self.funcParent = parent
         return self.funcWidget
 
+    def onWidgetStatusBar(self, parent):
+        self.statusBar = statusBar(rxTxCount=True)
+        return self.statusBar
+
     def onUiInitDone(self):
         paramObj = self.config
         self.receiveSettingsHex.setChecked(not paramObj["receiveAscii"])
@@ -367,7 +384,7 @@ class Plugin(Plugin_Base):
     def onSetColorChanged(self):
         self.config["color"] = self.receiveSettingsColor.isChecked()
 
-    def onSendHistoryIndexChanged(self):
+    def onSendHistoryIndexChanged(self, idx):
         self.sendArea.clear()
         self.sendArea.insertPlainText(self.sendHistory.currentText())
 
@@ -377,10 +394,20 @@ class Plugin(Plugin_Base):
         self.hintSignal.emit("info", _("OK"), _("History cleared!"))
 
 
-    def onSentFile(self, ok, path):
+    def onSent(self, ok, msg, length, path):
+        if ok:
+            self.statusBar.addTx(length)
+        else:
+            self.hintSignal.emit("error", _("Error"), _("Send data failed!") + " " + msg)
+
+    def onSentFile(self, ok, msg, length, path):
         print("file sent {}, path: {}".format('ok' if ok else 'fail', path))
-        self.sendFileButton.setText(_("Send file"))
-        self.sendFileButton.setDisabled(False)
+        if ok:
+            self.sendFileButton.setText(_("Send file"))
+            self.sendFileButton.setDisabled(False)
+            self.statusBar.addTx(length)
+        else:
+            self.hintSignal.emit("error", _("Error"), _("Send file failed!") + " " + msg)
 
     def setSaveLog(self):
         if self.saveLogCheckbox.isChecked():
@@ -508,7 +535,7 @@ class Plugin(Plugin_Base):
         else:
             self.sendFileButton.setDisabled(True)
             self.sendFileButton.setText(_("Sending file"))
-            self.send(file_path=filename, callback = lambda ok, msg: self.onSentFile(ok, filename))
+            self.send(file_path=filename, callback = lambda ok, msg, length, path: self.onSentFile(ok, msg, length, path))
 
 
     def scheduledSend(self):
@@ -550,7 +577,7 @@ class Plugin(Plugin_Base):
                         head = '{}: '.format(head.rstrip())
                     self.receiveUpdateSignal.emit(head, [sendStrsColored], self.configGlobal["encoding"])
                     self.sendRecord.insert(0, head + sendStr)
-                self.send(data_bytes=data)
+                self.send(data_bytes=data, callback = self.onSent)
                 if data_bytes:
                     data = str(data_bytes)
                 else:
@@ -636,24 +663,24 @@ class Plugin(Plugin_Base):
         colors = {
             b"0": None,
             b"30": "#000000",
-            b"31": "#ff0000",
-            b"32": "#008000",
-            b"33": "#f9a825",
-            b"34": "#0000ff",
-            b"35": "#9c27b0",
-            b"36": "#1b5e20",
-            b"37": "#000000",
+            b"31": "#f44336",
+            b"32": "#4caf50",
+            b"33": "#ffa000",
+            b"34": "#2196f3",
+            b"35": "#e85aad",
+            b"36": "#26c6da",
+            b"37": "#a1887f",
         }
         bgs = {
             b"0": None,
             b"40": "#000000",
-            b"41": "#ff0000",
-            b"42": "#008000",
-            b"43": "#f9a825",
-            b"44": "#0000ff",
-            b"45": "#9c27b0",
-            b"46": "#1b5e20",
-            b"47": "#000000",
+            b"41": "#f44336",
+            b"42": "#4caf50",
+            b"43": "#ffa000",
+            b"44": "#2196f3",
+            b"45": "#e85aad",
+            b"46": "#26c6da",
+            b"47": "#a1887f",
         }
         fmt = fmt[2:-1].split(b";")
         color = colors[b'0']
@@ -666,7 +693,35 @@ class Plugin(Plugin_Base):
         return color, bg
 
     def _texSplitByColor(self, text:bytes):
+        remain = b''
+        ignoreCodes = [rb'\x1b\[\?.*?h', rb'\x1b\[\?.*?l']
+        text = text.replace(b"\x1b[K", b"")
+        for code in ignoreCodes:
+            colorFmt = re.findall(code, text)
+            for fmt in colorFmt:
+                text = text.replace(fmt, b"")
         colorFmt = re.findall(rb'\x1b\[.*?m', text)
+        if text.endswith(b"\x1b"): # ***\x1b
+            text = text[:-1]
+            remain = b'\x1b'
+        elif text.endswith(b"\x1b["): # ***\x1b[
+            text = text[:-2]
+            remain = b'\x1b['
+        else: # ****\x1b[****, ****\x1b[****;****m
+            idx = -2
+            idx_remain = -1
+            while 1:
+                idx = text.find(b"\x1b[", len(text) - 10 + idx + 2) # \x1b[00;00m]
+                if idx < 0:
+                    break
+                remain = text[idx:]
+                idx_remain = idx
+            if len(remain) > 0:
+                match = re.findall(rb'\x1b\[.*?m', remain)  # ****\x1b[****;****m***
+                if len(match) > 0: # have full color format
+                    remain = b''
+                else:
+                    text = text[:idx_remain]
         plaintext = text
         for fmt in colorFmt:
             plaintext = plaintext.replace(fmt, b"")
@@ -683,17 +738,17 @@ class Plugin(Plugin_Base):
             colorStrs.append([self.lastColor, self.lastBg, text[p:]])
         else:
             colorStrs = [[self.lastColor, self.lastBg, text]]
-        return plaintext, colorStrs
+        return plaintext, colorStrs, remain
 
     def getColoredText(self, data_bytes, decoding=None):
-        plainText, coloredText = self._texSplitByColor(data_bytes)
+        plainText, coloredText, remain = self._texSplitByColor(data_bytes)
         if decoding:
             plainText = plainText.decode(encoding=decoding, errors="ignore")
             decodedColoredText = []
             for color, bg, text in coloredText:
                 decodedColoredText.append([color, bg, text.decode(encoding=decoding, errors="ignore")])
             coloredText = decodedColoredText
-        return plainText, coloredText
+        return plainText, coloredText, remain
 
     def bytes2String(self, data : bytes, showAsHex : bool, encoding="utf-8"):
         isHexString = False
@@ -701,7 +756,9 @@ class Plugin(Plugin_Base):
         if showAsHex:
             return True, utils.hexlify(data, ' ').decode(encoding=encoding), dataColored
         try:
-            dataPlain, dataColored = self.getColoredText(data, self.configGlobal["encoding"])
+            dataPlain, dataColore, remain = self.getColoredText(data, self.configGlobal["encoding"])
+            if remain:
+                dataPlain += remain.decode(encoding=self.configGlobal["encoding"], errors="ignore")
         except Exception:
             dataPlain = utils.hexlify(data, ' ').decode(encoding=encoding)
             isHexString = True
@@ -709,13 +766,11 @@ class Plugin(Plugin_Base):
 
     def clearReceiveBuffer(self):
         self.receiveArea.clear()
-        self.clearCountSignal.emit()
+        self.statusBar.clear()
 
     def onReceived(self, data : bytes):
         self.receivedData.append(data)
-        for plugin in self.connChilds:
-            if plugin.active:
-                plugin.onReceived(data)
+        self.statusBar.addRx(len(data))
         self.lock.release()
 
     def receiveDataProcess(self):
@@ -724,12 +779,13 @@ class Plugin(Plugin_Base):
         new_line = True
         logData = None
         buffer = b''
+        remain = b''
         while(not self.receiveProgressStop):
             logData = None
-            bytes = b''
             head = ""
             self.lock.acquire()
-            buffer += b"".join(self.receivedData)
+            new = b"".join(self.receivedData)
+            buffer += new
             self.receivedData = []
             # timeout, add new line
             if time.time() - timeLastReceive> self.config["receiveAutoLindefeedTime"]:
@@ -739,8 +795,6 @@ class Plugin(Plugin_Base):
                     else:
                         head += "\n"
                     new_line = True
-            if bytes:
-                timeLastReceive = time.time()
             data = ""
             # have data in buffer
             if len(buffer) > 0:
@@ -759,8 +813,8 @@ class Plugin(Plugin_Base):
                 # show as string, and need to render color, wait for \n or until timeout to ensure color flag in buffer
                 else:
                     if time.time() - timeLastReceive >  self.config["receiveAutoLindefeedTime"] or b'\n' in buffer:
-                        data, colorData = self.getColoredText(buffer, self.configGlobal["encoding"])
-                        buffer = b''
+                        data, colorData, remain = self.getColoredText(buffer, self.configGlobal["encoding"])
+                        buffer = remain
                 # add time receive head
                 # get data from buffer, now render
                 if data:
@@ -782,6 +836,8 @@ class Plugin(Plugin_Base):
                         new_line = False
                     self.receiveUpdateSignal.emit(head, [colorData], self.configGlobal["encoding"])
                     logData = head + data
+            if len(new) > 0:
+                timeLastReceive = time.time()
 
             while len(self.sendRecord) > 0:
                 self.onLog(self.sendRecord.pop())
